'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _block = require('../models/block');

var _block2 = _interopRequireDefault(_block);

var _character = require('../models/character');

var _character2 = _interopRequireDefault(_character);

var _document = require('../models/document');

var _document2 = _interopRequireDefault(_document);

var _inline = require('../models/inline');

var _inline2 = _interopRequireDefault(_inline);

var _mark = require('../models/mark');

var _mark2 = _interopRequireDefault(_mark);

var _state = require('../models/state');

var _state2 = _interopRequireDefault(_state);

var _text = require('../models/text');

var _text2 = _interopRequireDefault(_text);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Deserialize a JSON `object`.
 *
 * @param {Object} object
 * @return {Block}
 */

function deserialize(object) {
  switch (object.kind) {
    case 'block':
      return deserializeBlock(object);
    case 'document':
      return deserializeDocument(object);
    case 'inline':
      return deserializeInline(object);
    case 'mark':
      return deserializeMark(object);
    case 'range':
      return deserializeRange(object);
    case 'state':
      return deserializeState(object);
    case 'text':
      return deserializeText(object);
    default:
      {
        throw new Error('Unrecognized object kind "' + object.kind + '".');
      }
  }
}

/**
 * Deserialize a JSON `object` representing a `Block`.
 *
 * @param {Object} object
 * @return {Block}
 */

function deserializeBlock(object) {
  return _block2.default.create({
    key: object.key,
    type: object.type,
    data: object.data,
    isVoid: object.isVoid,
    nodes: _block2.default.createList(object.nodes.map(deserializeNode))
  });
}

/**
 * Deserialize a JSON `object` representing a `Document`.
 *
 * @param {Object} object
 * @return {Document}
 */

function deserializeDocument(object) {
  return _document2.default.create({
    nodes: _block2.default.createList(object.nodes.map(deserializeNode))
  });
}

/**
 * Deserialize a JSON `object` representing an `Inline`.
 *
 * @param {Object} object
 * @return {Inline}
 */

function deserializeInline(object) {
  return _inline2.default.create({
    key: object.key,
    type: object.type,
    data: object.data,
    isVoid: object.isVoid,
    nodes: _inline2.default.createList(object.nodes.map(deserializeNode))
  });
}

/**
 * Deserialize a JSON `object` representing a `Mark`.
 *
 * @param {Object} object
 * @return {Mark} mark
 */

function deserializeMark(object) {
  return _mark2.default.create(object);
}

/**
 * Deserialize a JSON object representing a `Node`.
 *
 * @param {Object} object
 * @return {Text}
 */

function deserializeNode(object) {
  switch (object.kind) {
    case 'block':
      return deserializeBlock(object);
    case 'document':
      return deserializeDocument(object);
    case 'inline':
      return deserializeInline(object);
    case 'text':
      return deserializeText(object);
    default:
      {
        throw new Error('Unrecognized node kind "' + object.kind + '".');
      }
  }
}

/**
 * Deserialize a JSON `object` representing a `Range`.
 *
 * @param {Object} object
 * @return {List}
 */

function deserializeRange(object) {
  return _character2.default.createList(object.text.split('').map(function (char) {
    return _character2.default.create({
      text: char,
      marks: _mark2.default.createSet(object.marks.map(deserializeMark))
    });
  }));
}

/**
 * Deserialize a JSON `object` representing a `State`.
 *
 * @param {Object} object
 * @return {State}
 */

function deserializeState(object) {
  return _state2.default.create({
    document: deserializeDocument(object.document)
  });
}

/**
 * Deserialize a JSON `object` representing a `Text`.
 *
 * @param {Object} object
 * @return {Text}
 */

function deserializeText(object) {
  return _text2.default.create({
    key: object.key,
    characters: object.ranges.reduce(function (characters, range) {
      return characters.concat(deserializeRange(range));
    }, _character2.default.createList())
  });
}

/**
 * Serialize a `model`.
 *
 * @param {Mixed} model
 * @return {Object}
 */

function serialize(model) {
  switch (model.kind) {
    case 'block':
      return serializeBlock(model);
    case 'document':
      return serializeDocument(model);
    case 'inline':
      return serializeInline(model);
    case 'mark':
      return serializeMark(model);
    case 'range':
      return serializeRange(model);
    case 'state':
      return serializeState(model);
    case 'text':
      return serializeText(model);
    default:
      {
        throw new Error('Unrecognized model kind "' + model.kind + '".');
      }
  }
}

/**
 * Serialize a `block` node.
 *
 * @param {Block} block
 * @return {Object}
 */

function serializeBlock(block) {
  return {
    data: block.data.toJSON(),
    key: block.key,
    kind: block.kind,
    isVoid: block.isVoid,
    type: block.type,
    nodes: block.nodes.toArray().map(serializeNode)
  };
}

/**
 * Serialize a `document`.
 *
 * @param {Document} document
 * @return {Object}
 */

function serializeDocument(document) {
  return {
    kind: document.kind,
    nodes: document.nodes.toArray().map(serializeNode)
  };
}

/**
 * Serialize an `inline` node.
 *
 * @param {Inline} inline
 * @return {Object}
 */

function serializeInline(inline) {
  return {
    data: inline.data.toJSON(),
    key: inline.key,
    kind: inline.kind,
    isVoid: inline.isVoid,
    type: inline.type,
    nodes: inline.nodes.toArray().map(serializeNode)
  };
}

/**
 * Serialize a `mark`.
 *
 * @param {Mark} mark
 * @return {Object} Object
 */

function serializeMark(mark) {
  return {
    data: mark.data.toJSON(),
    kind: mark.kind,
    type: mark.type
  };
}

/**
 * Serialize a `node`.
 *
 * @param {Node} node
 * @return {Object} object
 */

function serializeNode(node) {
  switch (node.kind) {
    case 'block':
      return serializeBlock(node);
    case 'document':
      return serializeDocument(node);
    case 'inline':
      return serializeInline(node);
    case 'text':
      return serializeText(node);
    default:
      {
        throw new Error('Unrecognized node kind "' + node.kind + '".');
      }
  }
}

/**
 * Serialize a `range`.
 *
 * @param {Range} range
 * @return {Object}
 */

function serializeRange(range) {
  return {
    kind: range.kind,
    text: range.text,
    marks: range.marks.toArray().map(serializeMark)
  };
}

/**
 * Serialize a `state`.
 *
 * @param {State} state
 * @return {Object}
 */

function serializeState(state) {
  return {
    document: serializeDocument(state.document),
    kind: state.kind
  };
}

/**
 * Serialize a `text` node.
 *
 * @param {Text} text
 * @return {Object}
 */

function serializeText(text) {
  return {
    key: text.key,
    kind: text.kind,
    ranges: text.getRanges().toArray().map(serializeRange)
  };
}

/**
 * Export.
 */

exports.default = {
  deserialize: deserialize,
  deserializeBlock: deserializeBlock,
  deserializeDocument: deserializeDocument,
  deserializeInline: deserializeInline,
  deserializeMark: deserializeMark,
  deserializeNode: deserializeNode,
  deserializeRange: deserializeRange,
  deserializeState: deserializeState,
  deserializeText: deserializeText,
  serialize: serialize,
  serializeBlock: serializeBlock,
  serializeDocument: serializeDocument,
  serializeInline: serializeInline,
  serializeMark: serializeMark,
  serializeNode: serializeNode,
  serializeRange: serializeRange,
  serializeState: serializeState,
  serializeText: serializeText
};